/*
 * Copyright (C) 2011 by egnite GmbH
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*
 * $Id: en5flasher.c 394 2013-07-18 16:43:28Z harald $
 */

#include <dev/board.h>
#include <dev/urom.h>

#include <sys/version.h>
#include <sys/timer.h>

#include <stdio.h>
#include <io.h>

#include "df_flasher.h"
#include "fc_flasher.h"

static void DisplayDesc(FLASH_DESC *desc)
{
    int i;

    printf("ID        : %08lx\n", desc->fl_id);
    printf("Total Size: %lu kBytes\n", desc->fl_size / 1024);
    printf("Page Size : %lu Bytes\n", desc->fl_page_size);
    for (i = 0; i < desc->fl_nb_plane; i++) {
        printf("Plane %2d  : %lu kBytes\n", i, desc->fl_plane[i] / 1024);
    }
    for (i = 0; i < desc->fl_nb_lock; i++) {
        printf("Lock %2d   : %lu kBytes\n", i, desc->fl_lock[i] / 1024);
    }
}

/*
 * Main application routine.
 */
int main(void)
{
    uint32_t baud = 115200;
    size_t df_pgsize;
    FLASH_DESC fldesc;

    /*
     * Register the UART device, open it, assign stdout to it and set
     * the baudrate.
     */
    NutRegisterDevice(&DEV_DEBUG, 0, 0);
    freopen(DEV_DEBUG_NAME, "w", stdout);
    _ioctl(_fileno(stdout), UART_SETSPEED, &baud);
    printf("\n\nEthernut 5 Flash Loader 1.2.0 (" __DATE__ " " __TIME__ ") - Nut/OS %s\n", NutVersionString());
    NutRegisterDevice(&devUrom, 0, 0);

    /* Set max. flash wait states. */
    outr(EEFC_FMR, 15 << EEFC_FWS_LSB);

    FlashReadDesc(&fldesc);
    DisplayDesc(&fldesc);
    FlashControllerFlasher("UROM:samboot.bin", 0, fldesc.fl_page_size);

    printf("GPNVM3 ");
    if (FlashGpvnmRead(3)) {
        puts("already set.");
    } else {
        FlashGpvnmWrite(3);
        if (FlashGpvnmRead(3)) {
            puts("set.");
        } else {
            puts("failed.");
        }
    }

    df_pgsize = DataFlashPageSize();
    if (df_pgsize != 512 && df_pgsize != 528) {
        printf("Error: Unexpected DataFlash page size of %d.\n", df_pgsize);
    } else {
        DataFlashFlasher("UROM:env-ethernut5.bin", 0x00021000, df_pgsize);
        DataFlashFlasher("UROM:u-boot-ethernut5.bin", 0x00042000, df_pgsize);
        DataFlashFlasher("UROM:uImage-ethernut5.itb", 0x000C6000, df_pgsize);
        DataFlashFlasher("UROM:nutos.bin", 0x0035A000, df_pgsize);
    }
    for (;;) {
        puts("Ready");
        NutSleep(60000);
    }
    return 0;
}
